INTEGER, PLUS, MINUS, MUL, DIV, EOF, LPAREN, RPAREN = 'INTEGER', 'PLUS', 'MINUS', 'MUL', 'DIV', 'EOF', 'LPAREN', 'RPAREN'  # 新增加左右括号常量


class Token:
    def __init__(self, value_type, value):
        self.value_type = value_type
        self.value = value

    def __str__(self):
        return 'Token({value_type},{value})'.format(value_type=self.value_type, value=self.value)

    def __repr__(self):
        return self.__str__()


class Lexer:
    def __init__(self, text):
        self.text = text
        self.position = 0
        self.current_char = self.text[self.position]

    def error(self):
        raise Exception('警告：错误的输入内容！')

    def skip_whitespace(self):
        while self.current_char is not None and self.current_char.isspace():
            self.advance()

    def long_integer(self):
        result = ''
        while self.current_char is not None and self.current_char.isdigit():
            result += self.current_char
            self.advance()
        return int(result)

    def advance(self):
        self.position += 1
        if self.position >= len(self.text):
            self.current_char = None
        else:
            self.current_char = self.text[self.position]

    def get_next_token(self):
        while self.current_char is not None:
            if self.current_char.isspace():
                self.skip_whitespace()
                continue
            if self.current_char.isdigit():
                return Token(INTEGER, self.long_integer())
            if self.current_char == '+':
                self.advance()
                return Token(PLUS, '+')
            if self.current_char == '-':
                self.advance()
                return Token(MINUS, '-')
            if self.current_char == '*':
                self.advance()
                return Token(MUL, '*')
            if self.current_char == '/':
                self.advance()
                return Token(DIV, '/')
            if self.current_char == '(':  # 新增
                self.advance()
                return Token(LPAREN, '(')
            if self.current_char == ')':  # 新增
                self.advance()
                return Token(RPAREN, ')')
            self.error()
        return Token(EOF, None)


class Interpreter:
    def __init__(self, lexer):
        self.lexer = lexer
        self.current_token = lexer.get_next_token()

    def eat(self, value_type):
        if self.current_token.value_type == value_type:
            self.current_token = self.lexer.get_next_token()
        else:
            self.lexer.error()

    def factor(self):
        current_token = self.current_token
        if current_token.value_type == INTEGER:  # 处理整数
            self.eat(INTEGER)
            return current_token.value
        elif current_token.value_type == LPAREN:  # 处理括号内表达式
            self.eat(LPAREN)  # 验证左括号
            result = self.expr()  # 计算括号内的表达式
            self.eat(RPAREN)  # 验证右括号
            return result  # 返回括号内表达式的值

    def term(self):
        result = self.factor()
        while self.current_token.value_type in (MUL, DIV):
            token = self.current_token
            if token.value_type == MUL:
                self.eat(MUL)
                result *= self.factor()
            elif token.value_type == DIV:
                self.eat(DIV)
                result //= self.factor()
        return result

    def expr(self):
        result = self.term()
        while self.current_token.value_type in (PLUS, MINUS):
            token = self.current_token
            if token.value_type == PLUS:
                self.eat(PLUS)
                result += self.term()
            elif token.value_type == MINUS:
                self.eat(MINUS)
                result -= self.term()
        return result


def main():
    while True:
        try:
            text = input('>>>')
        except EOFError:
            break
        if not text:
            continue
        lexer = Lexer(text)
        interpreter = Interpreter(lexer)
        result = interpreter.expr()
        print(text, '=', result)


if __name__ == '__main__':
    main()
